from lib import BaseTest


def sortLines(_, s):
    return "\n".join(sorted(s.split("\n")))


class ShowPackage1Test(BaseTest):
    """
    show package: regular show
    """
    fixtureDB = True
    outputMatchPrepare = sortLines
    runCmd = "aptly package show 'Name (% nginx-extras*)'"


class ShowPackage2Test(BaseTest):
    """
    show package: missing package
    """
    runCmd = "aptly package show 'Name (package-xx)'"


class ShowPackage3Test(BaseTest):
    """
    show package: by key
    """
    fixtureDB = True
    outputMatchPrepare = sortLines
    runCmd = "aptly package show nginx-full_1.2.1-2.2+wheezy2_amd64"


class ShowPackage4Test(BaseTest):
    """
    show package: with files
    """
    fixtureDB = True
    outputMatchPrepare = sortLines
    gold_processor = BaseTest.expand_environ
    runCmd = "aptly package show -with-files nginx-full_1.2.1-2.2+wheezy2_amd64"


class ShowPackage5Test(BaseTest):
    """
    show package: with inclusion
    """
    fixtureDB = True
    outputMatchPrepare = sortLines
    runCmd = "aptly package show -with-references nginx-full_1.2.1-2.2+wheezy2_amd64"


class ShowPackage6Test(BaseTest):
    """
    show package: with inclusion + more inclusions
    """
    fixtureDB = True
    fixtureCmds = [
        "aptly snapshot create snap1 from mirror wheezy-main",
        "aptly snapshot create snap2 from mirror wheezy-contrib",
        "aptly snapshot create snap3 from mirror wheezy-main-src",
        "aptly snapshot merge snap4 snap1 snap2 snap3",
        "aptly repo create repo1",
        "aptly repo import wheezy-main repo1 nginx",
    ]
    outputMatchPrepare = sortLines
    runCmd = "aptly package show -with-references nginx-full_1.2.1-2.2+wheezy2_amd64"


class ShowPackage7Test(BaseTest):
    """
    show package: with duplicates
    """
    fixtureCmds = [
        "aptly repo create a",
        "aptly repo create b",
        "aptly repo add a ${files}",
        "aptly repo add b ${testfiles}"
    ]
    outputMatchPrepare = sortLines
    runCmd = "aptly package show -with-references \"pyspi (0.6.1-1.3)\""


class ShowPackage8Test(BaseTest):
    """
    show package: with duplicates and direct reference
    """
    fixtureCmds = [
        "aptly repo create a",
        "aptly repo create b",
        "aptly repo add a ${files}",
        "aptly repo add b ${testfiles}"
    ]
    outputMatchPrepare = sortLines
    runCmd = "aptly package show -with-references pyspi_0.6.1-1.3_source"



from api_lib import APITest


class ReadyAPITest(APITest):
    """
    GET /ready
    """

    def check(self):
        resp = self.get("/api/ready")
        self.check_equal(resp.status_code, 200)

        readyStatus = "{\"Status\":\"Aptly is ready\"}"
        self.check_equal(readyStatus, resp.text)


class HealthyAPITest(APITest):
    """
    GET /healthy
    """

    def check(self):
        resp = self.get("/api/healthy")
        self.check_equal(resp.status_code, 200)

        healthyStatus = "{\"Status\":\"Aptly is healthy\"}"
        self.check_equal(healthyStatus, resp.text)



from lib import BaseTest


class DropRepo1Test(BaseTest):
    """
    drop repo: regular drop
    """
    fixtureCmds = [
        "aptly repo create repo1",
    ]
    runCmd = "aptly repo drop repo1"

    def check(self):
        self.check_output()
        self.check_cmd_output("aptly repo show repo1", "repo-show", expected_code=1)


class DropRepo2Test(BaseTest):
    """
    drop repo: in use by snapshots
    """
    fixtureCmds = [
        "aptly repo create repo2",
        "aptly repo add repo2 ${files}",
        "aptly snapshot create local from repo repo2",
    ]
    runCmd = "aptly repo drop repo2"
    expectedCode = 1


class DropRepo3Test(BaseTest):
    """
    drop repo: force
    """
    fixtureCmds = [
        "aptly repo create repo3",
        "aptly repo add repo3 ${files}",
        "aptly snapshot create local from repo repo3",
    ]
    runCmd = "aptly repo drop --force repo3"


class DropRepo4Test(BaseTest):
    """
    drop repo: no such repo
    """
    runCmd = "aptly repo drop repo4"
    expectedCode = 1


class DropRepo5Test(BaseTest):
    """
    drop repo: published
    """
    fixtureCmds = [
        "aptly repo create repo5",
        "aptly repo add repo5 ${files}",
        "aptly publish repo -skip-signing -distribution=squeeze repo5",
    ]
    runCmd = "aptly repo drop repo5"
    expectedCode = 1



# reuse existing tests:
from t06_publish.list import PublishList1Test, \
                             PublishList2Test, \
                             PublishList3Test, \
                             PublishList4Test, \
                             PublishList5Test

TEST_IGNORE = ["PublishList1Test", "PublishList2Test", "PublishList3Test", "PublishList4Test", "PublishList5Test"]


class PublishList1TestEtcd(PublishList1Test):
    """
    publish list: empty list
    """
    databaseType = "etcd"
    databaseUrl = "127.0.0.1:2379"


class PublishList2TestEtcd(PublishList2Test):
    """
    publish list: several repos list
    """
    databaseType = "etcd"
    databaseUrl = "127.0.0.1:2379"


class PublishList3TestEtcd(PublishList3Test):
    """
    publish list: several repos list, raw
    """
    databaseType = "etcd"
    databaseUrl = "127.0.0.1:2379"


class PublishList4TestEtcd(PublishList4Test):
    """
    publish list json: empty list
    """
    databaseType = "etcd"
    databaseUrl = "127.0.0.1:2379"


class PublishList5TestEtcd(PublishList5Test):
    """
    publish list json: several repos list
    """
    databaseType = "etcd"
    databaseUrl = "127.0.0.1:2379"



from lib import BaseTest


class RenameMirror1Test(BaseTest):
    """
    rename mirror: regular operations
    """
    fixtureDB = True
    runCmd = "aptly mirror rename wheezy-main wheezy-main-cool"

    def check(self):
        self.check_output()
        self.check_cmd_output("aptly mirror list", "mirror_list")


class RenameMirror2Test(BaseTest):
    """
    rename mirror: missing mirror
    """
    runCmd = "aptly mirror rename wheezy-main wheezy-main-cool"
    expectedCode = 1


class RenameMirror3Test(BaseTest):
    """
    rename mirror: already exists
    """
    fixtureDB = True
    runCmd = "aptly mirror rename wheezy-main wheezy-contrib"
    expectedCode = 1



from lib import BaseTest


class SearchPackage1Test(BaseTest):
    """
    search package: regular search
    """
    fixtureDB = True
    runCmd = "aptly package search '$$Architecture (i386), Name (% *-dev)'"


class SearchPackage2Test(BaseTest):
    """
    search package: missing package
    """
    runCmd = "aptly package search 'Name (package-xx)'"
    expectedCode = 1


class SearchPackage3Test(BaseTest):
    """
    search package: by key
    """
    fixtureDB = True
    runCmd = "aptly package search nginx-full_1.2.1-2.2+wheezy2_amd64"


class SearchPackage4Test(BaseTest):
    """
    search package: by dependency
    """
    fixtureDB = True
    runCmd = "aptly package search coreutils"


class SearchPackage5Test(BaseTest):
    """
    search package: with format
    """
    fixtureDB = True
    runCmd = "aptly package search -format='{{.Package}}#{{.Version}}' '$$Architecture (i386), Name (% *-dev)'"


class SearchPackage6Test(BaseTest):
    """
    search package: no query
    """
    fixtureDB = True
    runCmd = "aptly package search"



import re
from lib import BaseTest


def trimTrailingWhitespace(_, s):
    return re.sub(r'\s*$', '', s, flags=re.MULTILINE)


class DiffSnapshot1Test(BaseTest):
    """
    diff two snapshots: normal diff
    """
    fixtureDB = True
    fixtureCmds = [
        "aptly snapshot create snap1 from mirror wheezy-main",
        "aptly snapshot create snap2 from mirror wheezy-backports",
        "aptly snapshot pull snap1 snap2 snap3 'rsyslog (>= 7.4.4)'"
    ]
    runCmd = "aptly snapshot diff snap1 snap3"
    outputMatchPrepare = trimTrailingWhitespace


class DiffSnapshot2Test(BaseTest):
    """
    diff two snapshots: normal diff II
    """
    fixtureDB = True
    fixtureCmds = [
        "aptly snapshot create snap1 from mirror wheezy-main",
        "aptly snapshot create snap2 from mirror wheezy-backports",
    ]
    runCmd = "aptly snapshot diff snap1 snap2"
    outputMatchPrepare = trimTrailingWhitespace


class DiffSnapshot3Test(BaseTest):
    """
    diff two snapshots: normal diff II + only-matching
    """
    fixtureDB = True
    fixtureCmds = [
        "aptly snapshot create snap1 from mirror wheezy-main",
        "aptly snapshot create snap2 from mirror wheezy-backports",
    ]
    runCmd = "aptly snapshot diff -only-matching snap1 snap2"
    outputMatchPrepare = trimTrailingWhitespace


class DiffSnapshot4Test(BaseTest):
    """
    diff two snapshots: doesn't exist
    """
    fixtureDB = True
    fixtureCmds = [
        "aptly snapshot create snap1 from mirror wheezy-main",
    ]
    runCmd = "aptly snapshot diff -only-matching snap1 snap-no"
    expectedCode = 1


class DiffSnapshot5Test(BaseTest):
    """
    diff two snapshots: doesn't exist
    """
    fixtureDB = True
    fixtureCmds = [
        "aptly snapshot create snap2 from mirror wheezy-main",
    ]
    runCmd = "aptly snapshot diff -only-matching snap-no snap2"
    expectedCode = 1


class DiffSnapshot6Test(BaseTest):
    """
    diff two snapshots: identical snapshots
    """
    fixtureDB = True
    fixtureCmds = [
        "aptly snapshot create snap1 from mirror wheezy-main",
        "aptly snapshot create snap2 from mirror wheezy-main",
    ]
    runCmd = "aptly snapshot diff snap1 snap2"



import inspect
import os
import re
import shutil
import string

from lib import BaseTest


def filterOutSignature(_, s):
    return re.sub(r'Signature made .* using', '', s)


def filterOutRedirects(_, s):
    return re.sub(r'Following redirect to .+?\n', '', s)


class UpdateMirror1Test(BaseTest):
    """
    update mirrors: regular update
    """
    sortOutput = True
    longTest = False
    fixtureCmds = [
        "aptly -architectures=i386,amd64 mirror create --ignore-signatures varnish http://repo.aptly.info/system-tests/packagecloud.io/varnishcache/varnish30/debian/ wheezy main",
    ]
    runCmd = "aptly mirror update --ignore-signatures varnish"
    outputMatchPrepare = filterOutRedirects


class UpdateMirror2Test(BaseTest):
    """
    update mirrors: no such repo
    """
    runCmd = "aptly mirror update mirror-xyz"
    expectedCode = 1


class UpdateMirror3Test(BaseTest):
    """
    update mirrors: wrong checksum in release file
    """
    fixtureCmds = [
        "aptly mirror create --ignore-signatures failure ${url} hardy main",
    ]
    fixtureWebServer = "test_release"
    configOverride = {
        "downloadRetries": 0,
    }
    runCmd = "aptly mirror update --ignore-signatures failure"
    expectedCode = 1

    def gold_processor(self, gold):
        return string.Template(gold).substitute({'url': self.webServerUrl})


class UpdateMirror4Test(BaseTest):
    """
    update mirrors: wrong checksum in release file, but ignore
    """
    fixtureCmds = [
        "aptly mirror create --ignore-signatures failure ${url} hardy main",
    ]
    fixtureWebServer = "test_release"
    configOverride = {
        "downloadRetries": 0,
    }
    runCmd = "aptly mirror update -ignore-checksums --ignore-signatures failure"
    expectedCode = 1

    def gold_processor(self, gold):
        return string.Template(gold).substitute({'url': self.webServerUrl})


class UpdateMirror5Test(BaseTest):
    """
    update mirrors: wrong checksum in package
    """
    fixtureCmds = [
        "aptly mirror create --ignore-signatures failure ${url} hardy main",
    ]
    fixtureWebServer = "test_release2"
    configOverride = {
        "downloadRetries": 0,
    }
    runCmd = "aptly mirror update --ignore-signatures failure"
    expectedCode = 1

    def gold_processor(self, gold):
        return string.Template(gold).substitute({'url': self.webServerUrl})


class UpdateMirror6Test(BaseTest):
    """
    update mirrors: wrong checksum in package, but ignore
    """
    fixtureCmds = [
        "aptly mirror create --ignore-signatures failure ${url} hardy main",
    ]
    fixtureWebServer = "test_release2"
    configOverride = {
        "downloadRetries": 0,
    }

    runCmd = "aptly mirror update -ignore-checksums --ignore-signatures failure"

    def gold_processor(self, gold):
        return string.Template(gold).substitute({'url': self.webServerUrl})


class UpdateMirror7Test(BaseTest):
    """
    update mirrors: flat repository
    """
    sortOutput = True
    fixtureGpg = True
    fixtureCmds = [
            "aptly mirror create --keyring=aptlytest.gpg -architectures=amd64 flat http://repo.aptly.info/system-tests/cloud.r-project.org/bin/linux/debian bullseye-cran40/",
    ]
    runCmd = "aptly mirror update --keyring=aptlytest.gpg flat"
    outputMatchPrepare = filterOutSignature


class UpdateMirror8Test(BaseTest):
    """
    update mirrors: with sources (already in pool)
    """
    configOverride = {"max-tries": 1}

    fixtureGpg = True
    fixturePool = True
    fixtureCmds = [
        "aptly mirror create --keyring=aptlytest.gpg gnuplot-maverick-src http://repo.aptly.info/system-tests/ppa.launchpad.net/gladky-anton/gnuplot/ubuntu/ maverick",
    ]
    runCmd = "aptly mirror update --keyring=aptlytest.gpg gnuplot-maverick-src"
    outputMatchPrepare = filterOutSignature


class UpdateMirror9Test(BaseTest):
    """
    update mirrors: flat repository + sources
    """
    sortOutput = True
    fixtureGpg = True
    fixtureCmds = [
        "aptly mirror create --keyring=aptlytest.gpg -with-sources flat-src http://repo.aptly.info/system-tests/cloud.r-project.org/bin/linux/debian bullseye-cran40/",
    ]
    runCmd = "aptly mirror update --keyring=aptlytest.gpg flat-src"
    outputMatchPrepare = filterOutSignature


class UpdateMirror10Test(BaseTest):
    """
    update mirrors: filtered
    """
    sortOutput = True
    fixtureGpg = True
    fixtureCmds = [
        "aptly mirror create -keyring=aptlytest.gpg -with-sources -filter='!(Name (% r-*)), !($$PackageType (source))' flat-src http://repo.aptly.info/system-tests/cloud.r-project.org/bin/linux/debian bullseye-cran40/",
    ]
    runCmd = "aptly mirror update --keyring=aptlytest.gpg flat-src"
    outputMatchPrepare = filterOutSignature


class UpdateMirror11FTPTest(BaseTest):
    """
    update mirrors: update over FTP
    """
    configOverride = {"max-tries": 1}
    sortOutput = True
    longTest = False
    fixtureGpg = True
    requiresFTP = True
    fixtureCmds = [
        "aptly mirror create -keyring=aptlytest.gpg -filter='Priority (required), Name (% s*)' "
        "-architectures=i386 stretch-main http://repo.aptly.info/system-tests/snapshot.debian.org/archive/debian/20220201T025006Z/ stretch main",
    ]
    outputMatchPrepare = filterOutSignature
    runCmd = "aptly mirror update -keyring=aptlytest.gpg stretch-main"


class UpdateMirror12Test(BaseTest):
    """
    update mirrors: update with udebs
    """
    configOverride = {"max-tries": 1}
    sortOutput = True
    longTest = False
    fixtureGpg = True
    fixtureCmds = [
        "aptly -architectures=i386,amd64 mirror create -keyring=aptlytest.gpg -filter='$$Source (gnupg2)' -with-udebs stretch http://repo.aptly.info/system-tests/archive.debian.org/debian-archive/debian/ stretch main non-free",
    ]
    runCmd = "aptly mirror update -keyring=aptlytest.gpg stretch"
    outputMatchPrepare = filterOutSignature


class UpdateMirror13Test(BaseTest):
    """
    update mirrors: regular update with --skip-existing-packages option
    """
    sortOutput = True
    longTest = False
    fixtureCmds = [
        "aptly -architectures=i386,amd64 mirror create --ignore-signatures varnish http://repo.aptly.info/system-tests/packagecloud.io/varnishcache/varnish30/debian/ wheezy main",
    ]
    runCmd = "aptly mirror update --ignore-signatures --skip-existing-packages varnish"
    outputMatchPrepare = filterOutRedirects


class UpdateMirror14Test(BaseTest):
    """
    update mirrors: regular update with --skip-existing-packages option
    """
    sortOutput = True
    longTest = False
    fixtureCmds = [
        "aptly -architectures=i386,amd64 mirror create --ignore-signatures varnish http://repo.aptly.info/system-tests/packagecloud.io/varnishcache/varnish30/debian/ wheezy main",
        "aptly mirror update --ignore-signatures --skip-existing-packages varnish"
    ]
    runCmd = "aptly mirror update --ignore-signatures --skip-existing-packages varnish"
    outputMatchPrepare = filterOutRedirects


class UpdateMirror15Test(BaseTest):
    """
    update mirrors: update for mirror without MD5 checksums
    """
    # TODO spin up a Python server to serve that data from fixtures directory, instead of using bintray
    # e.g. python3 -m http.server --directory src/aptly/system/t04_mirror/test_release/
    # but that fixture seems to have the wrong hashes...
    skipTest = "Using deprecated service - bintray"
    sortOutput = True
    longTest = False
    fixtureCmds = [
        "aptly mirror create --ignore-signatures bintray https://dl.bintray.com/smira/deb/ ./",
        # TODO note the ./ is "flat" whereas putting "hardy" looks into dists/hardy
        # "aptly mirror create --ignore-signatures bintray http://localhost:8000/ hardy",
    ]
    runCmd = "aptly mirror update --ignore-signatures bintray"

    def check(self):
        super(UpdateMirror15Test, self).check()
        # check pool
        self.check_exists(
            'pool/c7/6b/4bd12fd92e4dfe1b55b18a67a669_libboost-program-options-dev_1.49.0.1_i386.deb')


class UpdateMirror16Test(BaseTest):
    """
    update mirrors: update for mirror without MD5 checksums but with file in pool on legacy MD5 location

    as mirror lacks MD5 checksum, file would be downloaded but not re-imported
    """
    skipTest = "Using deprecated service - bintray"
    sortOutput = True
    longTest = False
    fixtureCmds = [
        "aptly mirror create --ignore-signatures bintray https://dl.bintray.com/smira/deb/ ./",
    ]
    runCmd = "aptly mirror update --ignore-signatures bintray"

    def prepare(self):
        super(UpdateMirror16Test, self).prepare()

        os.makedirs(os.path.join(
            os.environ["HOME"], ".aptly", "pool", "00", "35"))

        shutil.copy(os.path.join(os.path.dirname(inspect.getsourcefile(BaseTest)), "files", "libboost-program-options-dev_1.49.0.1_i386.deb"),
                    os.path.join(os.environ["HOME"], ".aptly", "pool", "00", "35"))

    def check(self):
        super(UpdateMirror16Test, self).check()
        # check pool
        self.check_not_exists(
            'pool/c7/6b/4bd12fd92e4dfe1b55b18a67a669_libboost-program-options-dev_1.49.0.1_i386.deb')


class UpdateMirror17Test(BaseTest):
    """
    update mirrors: update for mirror but with file in pool on legacy MD5 location
    """
    sortOutput = True
    longTest = False
    fixtureCmds = [
        "aptly mirror create -ignore-signatures -architectures=i386 -filter=libboost-program-options-dev stretch http://repo.aptly.info/system-tests/archive.debian.org/debian-archive/debian stretch main",
    ]
    runCmd = "aptly mirror update -ignore-signatures stretch"

    def prepare(self):
        super(UpdateMirror17Test, self).prepare()

        os.makedirs(os.path.join(
            os.environ["HOME"], ".aptly", "pool", "e0", "bb"))

        shutil.copy(os.path.join(os.path.dirname(inspect.getsourcefile(BaseTest)), "files", "libboost-program-options-dev_1.62.0.1_i386.deb"),
                    os.path.join(os.environ["HOME"], ".aptly", "pool", "e0", "bb"))

    def check(self):
        super(UpdateMirror17Test, self).check()
        # check pool
        self.check_not_exists(
            'pool/db/a2/f225645a2a8bd8378e2f64bd1faa_libboost-program-options-dev_1.62.0.1_i386.deb')


class UpdateMirror18Test(BaseTest):
    """
    update mirrors: update for mirror but with file in pool on legacy MD5 location and disabled legacy path support
    """
    sortOutput = True
    longTest = False
    fixtureCmds = [
        "aptly mirror create -ignore-signatures -architectures=i386 -filter=libboost-program-options-dev stretch http://repo.aptly.info/system-tests/archive.debian.org/debian-archive/debian stretch main",
    ]
    runCmd = "aptly mirror update -ignore-signatures stretch"
    configOverride = {'skipLegacyPool': True}

    def prepare(self):
        super(UpdateMirror18Test, self).prepare()

        os.makedirs(os.path.join(
            os.environ["HOME"], ".aptly", "pool", "e0", "bb"))

        shutil.copy(os.path.join(os.path.dirname(inspect.getsourcefile(BaseTest)), "files", "libboost-program-options-dev_1.62.0.1_i386.deb"),
                    os.path.join(os.environ["HOME"], ".aptly", "pool", "e0", "bb"))

    def check(self):
        super(UpdateMirror18Test, self).check()
        # check pool
        self.check_exists(
            'pool/db/a2/f225645a2a8bd8378e2f64bd1faa_libboost-program-options-dev_1.62.0.1_i386.deb')


class UpdateMirror19Test(BaseTest):
    """
    update mirrors: correct matching of Release checksums
    """
    configOverride = {"max-tries": 1}
    longTest = False
    fixtureGpg = True
    fixtureCmds = [
        "aptly mirror create --keyring=aptlytest.gpg pagerduty http://repo.aptly.info/system-tests/packages.pagerduty.com/pdagent deb/"
    ]
    runCmd = "aptly mirror update --keyring=aptlytest.gpg pagerduty"
    outputMatchPrepare = filterOutSignature

    def output_processor(self, output):
        return "\n".join(line for line in self.ensure_utf8(output).split("\n") if ".deb" not in line)


class UpdateMirror20Test(BaseTest):
    """
    update mirrors: flat repository (internal GPG implementation)
    """
    sortOutput = True
    fixtureGpg = True
    configOverride = {"gpgProvider": "internal"}
    fixtureCmds = [
        "gpg --no-default-keyring --keyring aptlytest.gpg --export -o " + os.path.join(
            os.environ["HOME"], ".gnupg/aptlytest-gpg1.gpg"),
        "aptly mirror create --keyring=aptlytest-gpg1.gpg -architectures=amd64 --filter='r-cran-class' flat http://repo.aptly.info/system-tests/cloud.r-project.org/bin/linux/debian bullseye-cran40/",
    ]
    runCmd = "aptly mirror update --keyring=aptlytest-gpg1.gpg flat"
    outputMatchPrepare = filterOutSignature

    def teardown(self):
        self.run_cmd(["rm", "-f", os.path.join(os.environ["HOME"], ".gnupg/aptlytest-gpg1.gpg")])


class UpdateMirror21Test(BaseTest):
    """
    update mirrors: correct matching of Release checksums (internal pgp implementation)
    """
    longTest = False
    configOverride = {"gpgProvider": "internal", "max-tries": 1}
    fixtureGpg = True
    fixtureCmds = [
        "gpg --no-default-keyring --keyring aptlytest.gpg --export -o " + os.path.join(
            os.environ["HOME"], ".gnupg/aptlytest-gpg1.gpg"),
        "aptly mirror create --keyring=aptlytest-gpg1.gpg pagerduty http://repo.aptly.info/system-tests/packages.pagerduty.com/pdagent deb/"
    ]
    runCmd = "aptly mirror update --keyring=aptlytest-gpg1.gpg pagerduty"
    outputMatchPrepare = filterOutSignature

    def output_processor(self, output):
        return "\n".join(line for line in self.ensure_utf8(output).split("\n") if ".deb" not in line)

    def teardown(self):
        self.run_cmd(["rm", "-f", os.path.join(os.environ["HOME"], ".gnupg/aptlytest-gpg1.gpg")])


class UpdateMirror22Test(BaseTest):
    """
    update mirrors: SHA512 checksums only
    """
    configOverride = {"gpgProvider": "internal"}
    fixtureGpg = True
    fixtureCmds = [
        "gpg --no-default-keyring --keyring aptlytest.gpg --export -o " + os.path.join(
            os.environ["HOME"], ".gnupg/aptlytest-gpg1.gpg"),
        "aptly mirror create --keyring=aptlytest-gpg1.gpg --filter=nomatch libnvidia-container http://repo.aptly.info/system-tests/nvidia.github.io/libnvidia-container/stable/ubuntu16.04/amd64 ./"
    ]
    runCmd = "aptly mirror update --keyring=aptlytest-gpg1.gpg libnvidia-container"

    def outputMatchPrepare(self, s):
        return re.sub(r'Signature made .* using|Packages filtered: .* -> 0.', '', s)

    def teardown(self):
        self.run_cmd(["rm", "-f", os.path.join(os.environ["HOME"], ".gnupg/aptlytest-gpg1.gpg")])


class UpdateMirror23Test(BaseTest):
    """
    update mirrors: update with installer
    """
    configOverride = {"max-tries": 1}
    sortOutput = True
    longTest = False
    fixtureGpg = True
    fixtureCmds = [
        "aptly -architectures=s390x mirror create -keyring=aptlytest.gpg -filter='installer' -with-installer stretch http://repo.aptly.info/system-tests/archive.debian.org/debian-archive/debian/ stretch main non-free",
    ]
    runCmd = "aptly mirror update -keyring=aptlytest.gpg stretch"
    outputMatchPrepare = filterOutSignature


class UpdateMirror24Test(BaseTest):
    """
    update mirrors: update with installer with separate gpg file
    """
    configOverride = {"max-tries": 1}
    sortOutput = True
    longTest = False
    fixtureGpg = True
    fixtureCmds = [
        "aptly -architectures=amd64 mirror create -keyring=aptlytest.gpg -filter='installer' -with-installer trusty http://repo.aptly.info/system-tests/us.archive.ubuntu.com/ubuntu/ trusty main restricted",
    ]
    runCmd = "aptly mirror update -keyring=aptlytest.gpg trusty"
    outputMatchPrepare = filterOutSignature


class UpdateMirror25Test(BaseTest):
    """
    update mirrors: mirror with / in distribution
    """
    configOverride = {"max-tries": 1}
    sortOutput = True
    longTest = False
    fixtureGpg = True
    fixtureCmds = [
        "aptly -architectures='i386' mirror create -keyring=aptlytest.gpg -with-sources mirror19 http://repo.aptly.info/system-tests/archive.debian.org/debian-security/ stretch/updates main"
    ]
    runCmd = "aptly mirror update -keyring=aptlytest.gpg mirror19"
    outputMatchPrepare = filterOutSignature


class UpdateMirror26Test(BaseTest):
    """
    update mirrors: regular update, grab downloader
    """
    configOverride = {"downloader": "grab"}
    sortOutput = True
    longTest = False
    fixtureGpg = True
    fixtureCmds = [
        "aptly mirror create -architectures=i386 -keyring=aptlytest.gpg -filter=libboost-program-options-dev grab http://repo.aptly.info/system-tests/archive.debian.org/debian-archive/debian stretch main",
    ]
    runCmd = "aptly mirror update -downloader=grab -keyring=aptlytest.gpg grab"
    outputMatchPrepare = filterOutRedirects


class UpdateMirror27Test(BaseTest):
    """
    update mirrors: failing update, grab downloader
    """
    configOverride = {"downloader": "grab"}
    sortOutput = True
    longTest = False
    fixtureGpg = True
    fixtureCmds = [
        "aptly mirror create -architectures=amd64,i386 -keyring=aptlytest.gpg -filter=libboost-program-options-dev grab-fail http://repo.aptly.info/system-tests/archive.debian.org/debian-archive/debian stretch main",
    ]
    runCmd = "aptly mirror update -downloader=grab -keyring=aptlytest.gpg grab-fail"
    outputMatchPrepare = filterOutRedirects
    expectedCode = 1



from lib import BaseTest


class RenameRepo1Test(BaseTest):
    """
    rename repo: regular operations
    """
    fixtureCmds = [
        "aptly repo create repo1",
    ]
    runCmd = "aptly repo rename repo1 repo2"

    def check(self):
        self.check_output()
        self.check_cmd_output("aptly repo list", "repo_list")


class RenameRepo2Test(BaseTest):
    """
    rename repo: missing repo
    """
    runCmd = "aptly repo rename repo2 repo3"
    expectedCode = 1


class RenameRepo3Test(BaseTest):
    """
    rename repo: already exists
    """
    fixtureCmds = [
        "aptly repo create repo3",
        "aptly repo create repo4",
    ]
    runCmd = "aptly repo rename repo3 repo4"
    expectedCode = 1



from api_lib import APITest


class TaskAPITestSwaggerDocs(APITest):
    """
    GET /docs
    """

    def check(self):
        resp = self.get("/docs/doc.json")
        self.check_equal(resp.status_code, 200)

        resp = self.get("/docs/", allow_redirects=False)
        self.check_equal(resp.status_code, 301)

        resp = self.get("/docs/index.html")
        self.check_equal(resp.status_code, 200)



from api_lib import APITest


class TaskAPITestSwaggerDocs(APITest):
    """
    GET /docs
    """

    def check(self):
        resp = self.get("/api/storage")
        self.check_equal(resp.status_code, 200)



#!/usr/bin/env python

import glob
import importlib
import os
import inspect
import fnmatch
import re
import sys
from tempfile import mkdtemp
import traceback
import random
import subprocess
import time

from lib import BaseTest
from s3_lib import S3Test
from swift_lib import SwiftTest
from azure_lib import AzureTest
from api_lib import APITest
from fs_endpoint_lib import FileSystemEndpointTest
from testout import TestOut

try:
    from termcolor import colored
except ImportError:
    def colored(s, **kwargs):
        return s


PYTHON_MINIMUM_VERSION = (3, 9)


def natural_key(string_):
    """See https://blog.codinghorror.com/sorting-for-humans-natural-sort-order/"""
    return [int(s) if s.isdigit() else s for s in re.split(r'(\d+)', string_)]


def run(include_long_tests=False, capture_results=False, tests=None, filters=None, coverage_dir=None):
    """
    Run system test.
    """
    print(colored("\n Aptly System Tests\n====================\n", color="green", attrs=["bold"]))

    if not tests:
        tests = sorted(glob.glob("t*_*"), key=natural_key)
    fails = []
    numTests = numFailed = numSkipped = 0
    lastBase = None
    if not coverage_dir:
        coverage_dir = mkdtemp(suffix="aptly-coverage")

    for test in tests:
        orig_stdout = sys.stdout
        orig_stderr = sys.stderr

        # importlib.import_module(test)
        for fname in sorted(glob.glob(test + "/*.py"), key=natural_key):
            fname = os.path.splitext(os.path.basename(fname))[0]
            if fname == "__init__":
                continue

            testout = TestOut()
            sys.stdout = testout
            sys.stderr = testout

            try:
                testModule = importlib.import_module(test + "." + fname)
            except Exception as exc:
                orig_stdout.write(f"error importing: {test + '.' + fname}: {exc}\n")
                continue

            testignore = []
            if hasattr(testModule, "TEST_IGNORE"):
                testignore = testModule.TEST_IGNORE
            for name in sorted(dir(testModule), key=natural_key):
                if name in testignore:
                    continue

                o = getattr(testModule, name)
                if not (inspect.isclass(o) and issubclass(o, BaseTest) and o is not BaseTest and
                        o is not SwiftTest and o is not S3Test and o is not AzureTest and
                        o is not APITest and o is not FileSystemEndpointTest):
                    continue

                testout.clear()
                start_time = time.time()

                newBase = o.__bases__[0]
                if lastBase is not None and lastBase is not newBase:
                    lastBase.shutdown_class()

                lastBase = newBase

                if filters:
                    matches = False

                    for filt in filters:
                        if fnmatch.fnmatch(o.__name__, filt):
                            matches = True
                            break

                    if not matches:
                        continue

                orig_stdout.write("· %-13s ➔ %-48s ... " % (test, colored(o.__name__, color="yellow", attrs=["bold"])))
                orig_stdout.flush()

                t = o()

                if t.longTest and not include_long_tests or not t.fixture_available() or t.skipTest:
                    numSkipped += 1
                    msg = 'SKIP'
                    if t.skipTest and t.skipTest is not True:
                        # If we have a reason to skip, print it
                        msg += ': ' + t.skipTest
                    orig_stdout.write(colored(msg + "\n", color="yellow"))
                    continue

                numTests += 1

                failed = False
                t.captureResults = capture_results
                t.coverage_dir = coverage_dir
                typ = None
                val = None
                tb = None
                try:
                    t.test()
                except Exception:
                    typ, val, tb = sys.exc_info()
                    failed = True

                end_time = time.time()
                execution_time = int(end_time - start_time) + 1
                minutes = execution_time // 60
                seconds = execution_time % 60
                if minutes > 0:
                    minutes = f"{minutes}m"
                    if seconds < 10:
                        seconds = f"0{seconds}"
                else:
                    minutes = "  "
                    if seconds < 10:
                        seconds = f" {seconds}"
                duration = f"{minutes}{seconds}s"

                if failed:
                    numFailed += 1
                    fails.append((test, t, typ, val, tb, testModule))
                    orig_stdout.write(colored("\b\b\b\bFAIL", color="red", attrs=["bold"]) + f"   {duration}\n")

                    orig_stdout.write(testout.get_contents())
                    traceback.print_exception(typ, val, tb, file=orig_stdout)
                else:
                    orig_stdout.write(colored("\b\b\b\bOK", color="green", attrs=["bold"]) + f"   {duration}\n")

                t.shutdown()

        sys.stdout = orig_stdout
        sys.stderr = orig_stderr

    if lastBase is not None:
        lastBase.shutdown_class()

    print("\nCOVERAGE_RESULTS: %s" % coverage_dir)

    print(f"TESTS: {numTests}    ",
          colored(f"SUCCESS: {numTests - numFailed}    ", color="green", attrs=["bold"]) if numFailed == 0 else
          f"SUCCESS: {numTests - numFailed}    ",
          colored(f"FAIL: {numFailed}    ", color="red", attrs=["bold"]) if numFailed > 0 else "FAIL: 0    ",
          colored(f"SKIP: {numSkipped}", color="yellow", attrs=["bold"]) if numSkipped > 0 else "SKIP: 0")
    print()

    if len(fails) > 0:
        print(colored("FAILURES (%d):" % (len(fails), ), color="red", attrs=["bold"]))

        for (test, t, typ, val, tb, testModule) in fails:
            doc = t.__doc__ or ''
            print(" - %s: %s %s" % (test, colored(t.__class__.__name__, color="yellow", attrs=["bold"]),
                                    testModule.__name__ + ": " + doc.strip()))
        print()
        sys.exit(1)


if __name__ == "__main__":
    try:
        os.environ['APTLY_VERSION'] = os.popen("make -s version").read().strip()
    except BaseException as e:
        print("Failed to capture current version: ", e)

    if sys.version_info < PYTHON_MINIMUM_VERSION:
        raise RuntimeError(f'Tests require Python {PYTHON_MINIMUM_VERSION} or higher.')

    output = subprocess.check_output(['gpg', '--version'], text=True)
    if not output.startswith('gpg (GnuPG) 2'):
        raise RuntimeError('Tests require gpg v2')

    output = subprocess.check_output(['gpgv', '--version'], text=True)
    if not output.startswith('gpgv (GnuPG) 2'):
        raise RuntimeError('Tests require gpgv v2')

    os.chdir(os.path.realpath(os.path.dirname(sys.argv[0])))
    random.seed()
    include_long_tests = False
    capture_results = False
    coverage_dir = None
    tests = None
    args = sys.argv[1:]

    while len(args) > 0 and args[0].startswith("--"):
        if args[0] == "--long":
            include_long_tests = True
        elif args[0] == "--capture":
            capture_results = True
        elif args[0] == "--coverage-dir":
            coverage_dir = args[1]
            args = args[1:]

        args = args[1:]

    tests = []
    filters = []

    for arg in args:
        if arg.startswith('t'):
            tests.append(arg)
        else:
            filters.append(arg)

    run(include_long_tests, capture_results, tests, filters, coverage_dir)



from api_lib import APITest


class FilesAPITestUpload(APITest):
    """
    POST /files/:dir
    """

    def check(self):
        d = self.random_name()
        resp = self.upload("/api/files/" + d, "pyspi_0.6.1-1.3.dsc")
        self.check_equal(resp.status_code, 200)
        self.check_equal(resp.json(), [d + '/pyspi_0.6.1-1.3.dsc'])
        self.check_exists("upload/" + d + '/pyspi_0.6.1-1.3.dsc')


class FilesAPITestUploadMulti(APITest):
    """
    POST /files/:dir, GET /files/:dir multi files
    """

    def check(self):
        d = self.random_name()

        self.check_equal(self.get("/api/files/" + d).status_code, 404)

        resp = self.upload("/api/files/" + d, "pyspi_0.6.1-1.3.dsc", "pyspi_0.6.1-1.3.diff.gz", "pyspi_0.6.1.orig.tar.gz")
        self.check_equal(resp.status_code, 200)
        self.check_equal(sorted(resp.json()),
                         [d + '/pyspi_0.6.1-1.3.diff.gz', d + '/pyspi_0.6.1-1.3.dsc', d + '/pyspi_0.6.1.orig.tar.gz'])
        self.check_exists("upload/" + d + '/pyspi_0.6.1-1.3.dsc')
        self.check_exists("upload/" + d + '/pyspi_0.6.1-1.3.diff.gz')
        self.check_exists("upload/" + d + '/pyspi_0.6.1.orig.tar.gz')

        resp = self.get("/api/files/" + d)
        self.check_equal(resp.status_code, 200)
        self.check_equal(sorted(resp.json()),
                         ['pyspi_0.6.1-1.3.diff.gz', 'pyspi_0.6.1-1.3.dsc', 'pyspi_0.6.1.orig.tar.gz'])


class FilesAPITestList(APITest):
    """
    GET /files/
    """

    def check(self):
        d1, d2, d3 = self.random_name(), self.random_name(), self.random_name()

        resp = self.get("/api/files")
        self.check_equal(resp.status_code, 200)
        self.check_equal(resp.json(), [])

        self.check_equal(self.upload("/api/files/" + d1, "pyspi_0.6.1-1.3.dsc").status_code, 200)
        self.check_equal(self.upload("/api/files/" + d2, "pyspi_0.6.1-1.3.dsc").status_code, 200)
        self.check_equal(self.upload("/api/files/" + d3, "pyspi_0.6.1-1.3.dsc").status_code, 200)

        resp = self.get("/api/files")
        self.check_equal(resp.status_code, 200)
        self.check_equal(sorted(resp.json()), sorted([d1, d2, d3]))


class FilesAPITestDelete(APITest):
    """
    DELETE /files/:dir, DELETE /files/:dir/:name
    """

    def check(self):
        d1, d2 = self.random_name(), self.random_name()

        self.check_equal(self.get("/api/files").json(), [])
        self.check_equal(self.delete("/api/files/" + d1).status_code, 200)
        self.check_equal(self.delete("/api/files/" + d1 + "/" + "pyspi_0.6.1-1.3.dsc").status_code, 200)

        self.check_equal(self.upload("/api/files/" + d1, "pyspi_0.6.1-1.3.dsc").status_code, 200)
        self.check_equal(self.upload("/api/files/" + d2, "pyspi_0.6.1-1.3.dsc").status_code, 200)

        self.check_equal(self.delete("/api/files/" + d1).status_code, 200)
        self.check_equal(self.get("/api/files").json(), [d2])

        self.check_equal(self.delete("/api/files/" + d2 + "/" + "no-such-file").status_code, 200)
        self.check_equal(self.get("/api/files/" + d2).json(), ["pyspi_0.6.1-1.3.dsc"])

        self.check_equal(self.delete("/api/files/" + d2 + "/" + "pyspi_0.6.1-1.3.dsc").status_code, 200)
        self.check_equal(self.get("/api/files").json(), [d2])
        self.check_equal(self.get("/api/files/" + d2).json(), [])


class FilesAPITestSecurity(APITest):
    """
    delete & upload security
    """

    def check(self):
        self.check_equal(self.delete("/api/files/.").status_code, 404)
        self.check_equal(self.delete("/api/files").status_code, 404)
        self.check_equal(self.delete("/api/files/").status_code, 404)
        self.check_equal(self.delete("/api/files/../.").status_code, 404)
        self.check_equal(self.delete("/api/files/./..").status_code, 404)
        self.check_equal(self.delete("/api/files/dir/..").status_code, 404)



import os
import inspect
from lib import BaseTest


def changesRemove(_, s):
    return s.replace(os.path.join(os.path.dirname(inspect.getsourcefile(BaseTest)), "changes"), "")


class CreateRepo1Test(BaseTest):
    """
    create local repo: regular repo
    """
    runCmd = "aptly repo create repo1"

    def check(self):
        self.check_output()
        self.check_cmd_output("aptly repo show repo1", "repo_show")


class CreateRepo2Test(BaseTest):
    """
    create local repo: regular repo with comment & publishing defaults
    """
    runCmd = "aptly repo create -comment=Repository2 -distribution=maverick -component=non-free repo2"

    def check(self):
        self.check_output()
        self.check_cmd_output("aptly repo show repo2", "repo_show")


class CreateRepo3Test(BaseTest):
    """
    create local repo: duplicate name
    """
    fixtureCmds = ["aptly repo create repo3"]
    runCmd = "aptly repo create -comment=Repository3 repo3"
    expectedCode = 1


class CreateRepo4Test(BaseTest):
    """
    create local repo: with uploaders.json
    """
    runCmd = "aptly repo create -uploaders-file=${changes}/uploaders2.json repo4"

    def check(self):
        self.check_output()
        self.check_cmd_output("aptly repo show repo4", "repo_show")


class CreateRepo5Test(BaseTest):
    """
    create local repo: with broken uploaders.json
    """
    runCmd = "aptly repo create -uploaders-file=${changes}/uploaders3.json repo5"
    expectedCode = 1


class CreateRepo6Test(BaseTest):
    """
    create local repo: with missing uploaders.json
    """
    runCmd = "aptly repo create -uploaders-file=${changes}/uploaders-not-found.json repo6"
    expectedCode = 1
    outputMatchPrepare = changesRemove


class CreateRepo7Test(BaseTest):
    """
    create local repo: from snapshot
    """
    fixtureCmds = [
        "aptly repo create repo1",
        "aptly repo add repo1 ${files}",
        "aptly snapshot create snap1 from repo repo1",
    ]
    runCmd = "aptly repo create repo2 from snapshot snap1"

    def check(self):
        self.check_output()
        self.check_cmd_output("aptly repo show -with-packages repo2", "repo_show")


class CreateRepo8Test(BaseTest):
    """
    create local repo: from missing snapshot
    """
    runCmd = "aptly repo create repo2 from snapshot snap-missing"
    expectedCode = 1



from lib import BaseTest
import re


def sortLines(_, output):
    return "\n".join(sorted(output.split("\n")))


class PullSnapshot1Test(BaseTest):
    """
    pull snapshot: simple conditions
    """
    fixtureDB = True
    fixtureCmds = [
        "aptly snapshot create snap1 from mirror wheezy-main",
        "aptly snapshot create snap2 from mirror wheezy-non-free",
    ]
    runCmd = "aptly snapshot pull snap1 snap2 snap3 mame unrar"
    outputMatchPrepare = sortLines

    def check(self):
        def remove_created_at(s):
            return re.sub(r"Created At: [0-9:A-Za-z -]+\n", "", s)

        self.check_output()
        self.check_cmd_output("aptly snapshot show -with-packages snap3", "snapshot_show", match_prepare=remove_created_at)


class PullSnapshot2Test(BaseTest):
    """
    pull snapshot: play with versions
    """
    fixtureDB = True
    fixtureCmds = [
        "aptly snapshot create snap1 from mirror wheezy-main",
        "aptly snapshot create snap2 from mirror wheezy-backports",
    ]
    runCmd = "aptly snapshot pull snap1 snap2 snap3 'rsyslog (>= 7.4.4)'"
    outputMatchPrepare = sortLines

    def check(self):
        def remove_created_at(s):
            return re.sub(r"Created At: [0-9:A-Za-z -]+\n", "", s)

        self.check_output()
        self.check_cmd_output("aptly snapshot show -with-packages snap3", "snapshot_show", match_prepare=remove_created_at)


class PullSnapshot3Test(BaseTest):
    """
    pull snapshot: play with versions + no-deps
    """
    fixtureDB = True
    fixtureCmds = [
        "aptly snapshot create snap1 from mirror wheezy-main",
        "aptly snapshot create snap2 from mirror wheezy-backports",
    ]
    runCmd = "aptly snapshot pull -no-deps snap1 snap2 snap3 'rsyslog (>= 7.4.4)'"
    outputMatchPrepare = sortLines

    def check(self):
        def remove_created_at(s):
            return re.sub(r"Created At: [0-9:A-Za-z -]+\n", "", s)

        self.check_output()
        self.check_cmd_output("aptly snapshot show -with-packages snap3", "snapshot_show", match_prepare=remove_created_at)


class PullSnapshot4Test(BaseTest):
    """
    pull snapshot: dry-run
    """
    fixtureDB = True
    fixtureCmds = [
        "aptly snapshot create snap1 from mirror wheezy-main",
        "aptly snapshot create snap2 from mirror wheezy-backports",
    ]
    runCmd = "aptly snapshot pull -dry-run snap1 snap2 snap3 'rsyslog (>= 7.4.4)'"
    outputMatchPrepare = sortLines

    def check(self):
        self.check_output()
        self.check_cmd_output("aptly snapshot list", "snapshot_list")


class PullSnapshot5Test(BaseTest):
    """
    pull snapshot: no such snapshot
    """
    fixtureDB = True
    fixtureCmds = [
        "aptly snapshot create snap1 from mirror wheezy-main",
        "aptly snapshot create snap2 from mirror wheezy-backports",
    ]
    runCmd = "aptly snapshot pull snap1 snap-no snap3 'rsyslog (>= 7.4.4)'"
    expectedCode = 1


class PullSnapshot6Test(BaseTest):
    """
    pull snapshot: no such snapshot
    """
    fixtureDB = True
    fixtureCmds = [
        "aptly snapshot create snap1 from mirror wheezy-main",
        "aptly snapshot create snap2 from mirror wheezy-backports",
    ]
    runCmd = "aptly snapshot pull snap-no snap2 snap3 'rsyslog (>= 7.4.4)'"
    expectedCode = 1


class PullSnapshot7Test(BaseTest):
    """
    pull snapshot: duplicate snapshot
    """
    fixtureDB = True
    fixtureCmds = [
        "aptly snapshot create snap1 from mirror wheezy-main",
        "aptly snapshot create snap2 from mirror wheezy-backports",
    ]
    runCmd = "aptly snapshot pull snap1 snap2 snap1 'rsyslog (>= 7.4.4)'"
    expectedCode = 1
    outputMatchPrepare = sortLines


class PullSnapshot8Test(BaseTest):
    """
    pull snapshot: missing dependencies
    """
    fixtureDB = True
    fixtureCmds = [
        "aptly snapshot create snap1 from mirror wheezy-main",
        "aptly snapshot create snap2 from mirror wheezy-non-free",
    ]
    runCmd = "aptly snapshot pull snap1 snap2 snap3 lunar-landing 'mars-landing (>= 1.0)'"
    outputMatchPrepare = sortLines

    def check(self):
        def remove_created_at(s):
            return re.sub(r"Created At: [0-9:A-Za-z -]+\n", "", s)

        self.check_output()
        self.check_cmd_output("aptly snapshot show --with-packages snap3", "snapshot_show", match_prepare=remove_created_at)


class PullSnapshot9Test(BaseTest):
    """
    pull snapshot: follow sources
    """
    fixtureDB = True
    fixtureCmds = [
        "aptly snapshot create snap1 from mirror wheezy-main",
        "aptly snapshot create snap2 from mirror wheezy-backports-src",
    ]
    runCmd = "aptly -dep-follow-source snapshot pull snap1 snap2 snap3 'rsyslog (>= 7.4.4)'"

    def check(self):
        def remove_created_at(s):
            return re.sub(r"Created At: [0-9:A-Za-z -]+\n", "", s)

        self.check_output()
        self.check_cmd_output("aptly snapshot show --with-packages snap3", "snapshot_show", match_prepare=remove_created_at)


class PullSnapshot10Test(BaseTest):
    """
    pull snapshot: follow sources + replace sources
    """
    fixtureDB = True
    fixtureCmds = [
        "aptly snapshot create snap1 from mirror wheezy-main-src",
        "aptly snapshot create snap2 from mirror wheezy-backports-src",
    ]
    runCmd = "aptly -dep-follow-source snapshot pull snap1 snap2 snap3 'rsyslog (>= 7.4.4)'"

    def check(self):
        def remove_created_at(s):
            return re.sub(r"Created At: [0-9:A-Za-z -]+\n", "", s)

        self.check_output()
        self.check_cmd_output("aptly snapshot show --with-packages snap3", "snapshot_show", match_prepare=remove_created_at)


class PullSnapshot11Test(BaseTest):
    """
    pull snapshot: -no-remove
    """
    fixtureDB = True
    fixtureCmds = [
        "aptly snapshot create snap1 from mirror wheezy-main",
        "aptly snapshot create snap2 from mirror wheezy-backports",
    ]
    runCmd = "aptly snapshot pull -no-remove snap1 snap2 snap3 'rsyslog (>= 7.4.4)'"
    outputMatchPrepare = sortLines

    def check(self):
        def remove_created_at(s):
            return re.sub(r"Created At: [0-9:A-Za-z -]+\n", "", s)

        self.check_output()
        self.check_cmd_output("aptly snapshot show -with-packages snap3", "snapshot_show", match_prepare=remove_created_at)


class PullSnapshot12Test(BaseTest):
    """
    pull snapshot: latest version is pulled by default
    """
    fixtureDB = True
    fixtureCmds = [
        "aptly snapshot create empty empty",
        "aptly snapshot create sensu from mirror sensu",
    ]
    runCmd = "aptly snapshot pull -architectures=amd64,i386 empty sensu destination sensu"


class PullSnapshot13Test(BaseTest):
    """
    pull snapshot: pull all versions
    """
    fixtureDB = True
    fixtureCmds = [
        "aptly snapshot create empty empty",
        "aptly snapshot create sensu from mirror sensu",
    ]
    runCmd = "aptly snapshot pull -architectures=amd64,i386 -all-matches empty sensu destination sensu"


class PullSnapshot14Test(BaseTest):
    """
    pull snapshot: pull with query
    """
    fixtureDB = True
    fixtureCmds = [
        "aptly snapshot create empty empty",
        "aptly snapshot create sensu from mirror sensu",
    ]
    runCmd = "aptly snapshot pull -architectures=amd64,i386 -all-matches empty sensu destination 'sensu (>0.12)' 'sensu (<0.9.6)'"


class PullSnapshot15Test(BaseTest):
    """
    pull snapshot: verbose logs
    """
    fixtureDB = True
    fixtureCmds = [
        "aptly snapshot create snap1 from mirror wheezy-main",
        "aptly snapshot create snap2 from mirror wheezy-backports",
    ]
    runCmd = "aptly snapshot pull -dep-verbose-resolve snap1 snap2 snap3 'rsyslog (>= 7.4.4)'"
    outputMatchPrepare = sortLines



from azure_lib import AzureTest


def strip_processor(output):
    return '\n'.join(
        [
            l
            for l in output.split('\n')
            if not l.startswith(' ') and not l.startswith('Date:')
        ]
    )


class AzurePublish1Test(AzureTest):
    """
    publish to Azure: from repo
    """

    fixtureCmds = [
        'aptly repo create -distribution=maverick local-repo',
        'aptly repo add local-repo ${files}',
        'aptly repo remove local-repo libboost-program-options-dev_1.62.0.1_i386',
    ]
    runCmd = 'aptly publish repo -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec local-repo azure:test1:'

    def check(self):
        super(AzurePublish1Test, self).check()

        self.check_exists('public/dists/maverick/InRelease')
        self.check_exists('public/dists/maverick/Release')
        self.check_exists('public/dists/maverick/Release.gpg')

        self.check_exists('public/dists/maverick/main/binary-i386/Packages')
        self.check_exists('public/dists/maverick/main/binary-i386/Packages.gz')
        self.check_exists('public/dists/maverick/main/binary-i386/Packages.bz2')
        self.check_exists('public/dists/maverick/main/source/Sources')
        self.check_exists('public/dists/maverick/main/source/Sources.gz')
        self.check_exists('public/dists/maverick/main/source/Sources.bz2')

        self.check_exists('public/pool/main/p/pyspi/pyspi_0.6.1-1.3.dsc')
        self.check_exists('public/pool/main/p/pyspi/pyspi_0.6.1-1.3.diff.gz')
        self.check_exists('public/pool/main/p/pyspi/pyspi_0.6.1.orig.tar.gz')
        self.check_exists('public/pool/main/p/pyspi/pyspi-0.6.1-1.3.stripped.dsc')
        self.check_exists(
            'public/pool/main/b/boost-defaults/libboost-program-options-dev_1.49.0.1_i386.deb'
        )

        # # verify contents except of sums
        self.check_file_contents(
            'public/dists/maverick/Release', 'release', match_prepare=strip_processor
        )
        self.check_file_contents(
            'public/dists/maverick/main/source/Sources',
            'sources',
            match_prepare=lambda s: '\n'.join(sorted(s.split('\n'))),
        )
        self.check_file_contents(
            'public/dists/maverick/main/binary-i386/Packages',
            'binary',
            match_prepare=lambda s: '\n'.join(sorted(s.split('\n'))),
        )


class AzurePublish2Test(AzureTest):
    """
    publish to Azure: publish update removed some packages
    """

    fixtureCmds = [
        'aptly repo create -distribution=maverick local-repo',
        'aptly repo add local-repo ${files}/',
        'aptly repo remove local-repo libboost-program-options-dev_1.62.0.1_i386',
        'aptly publish repo -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec local-repo azure:test1:',
        'aptly repo remove local-repo pyspi',
    ]
    runCmd = 'aptly publish update -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec maverick azure:test1:'

    def check(self):
        super(AzurePublish2Test, self).check()

        self.check_exists('public/dists/maverick/InRelease')
        self.check_exists('public/dists/maverick/Release')
        self.check_exists('public/dists/maverick/Release.gpg')

        self.check_exists('public/dists/maverick/main/binary-i386/Packages')
        self.check_exists('public/dists/maverick/main/binary-i386/Packages.gz')
        self.check_exists('public/dists/maverick/main/binary-i386/Packages.bz2')
        self.check_exists('public/dists/maverick/main/source/Sources')
        self.check_exists('public/dists/maverick/main/source/Sources.gz')
        self.check_exists('public/dists/maverick/main/source/Sources.bz2')

        self.check_not_exists('public/pool/main/p/pyspi/pyspi_0.6.1-1.3.dsc')
        self.check_not_exists('public/pool/main/p/pyspi/pyspi_0.6.1-1.3.diff.gz')
        self.check_not_exists('public/pool/main/p/pyspi/pyspi_0.6.1.orig.tar.gz')
        self.check_not_exists('public/pool/main/p/pyspi/pyspi-0.6.1-1.3.stripped.dsc')
        self.check_exists(
            'public/pool/main/b/boost-defaults/libboost-program-options-dev_1.49.0.1_i386.deb'
        )

        # verify contents except of sums
        self.check_file_contents(
            'public/dists/maverick/Release', 'release', match_prepare=strip_processor
        )
        self.check_file_contents(
            'public/dists/maverick/main/source/Sources',
            'sources',
            match_prepare=lambda s: '\n'.join(sorted(s.split('\n'))),
        )
        self.check_file_contents(
            'public/dists/maverick/main/binary-i386/Packages',
            'binary',
            match_prepare=lambda s: '\n'.join(sorted(s.split('\n'))),
        )


class AzurePublish3Test(AzureTest):
    """
    publish to Azure: publish switch - removed some packages
    """

    fixtureDB = True
    fixturePool = True
    fixtureCmds = [
        'aptly snapshot create snap1 from mirror gnuplot-maverick',
        'aptly snapshot create snap2 empty',
        'aptly snapshot pull -no-deps -architectures=i386,amd64 snap2 snap1 snap3 gnuplot-x11',
        'aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec -distribution=maverick snap1 azure:test1:',
    ]
    runCmd = 'aptly publish switch -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec maverick azure:test1: snap3'

    def check(self):
        super(AzurePublish3Test, self).check()

        self.check_exists('public/dists/maverick/InRelease')
        self.check_exists('public/dists/maverick/Release')
        self.check_exists('public/dists/maverick/Release.gpg')

        self.check_exists('public/dists/maverick/main/binary-i386/Packages.gz')
        self.check_exists('public/dists/maverick/main/binary-i386/Packages.bz2')
        self.check_exists('public/dists/maverick/main/binary-amd64/Packages')
        self.check_exists('public/dists/maverick/main/binary-amd64/Packages.gz')
        self.check_exists('public/dists/maverick/main/binary-amd64/Packages.bz2')

        self.check_exists(
            'public/pool/main/g/gnuplot/gnuplot-x11_4.6.1-1~maverick2_i386.deb'
        )
        self.check_exists(
            'public/pool/main/g/gnuplot/gnuplot-x11_4.6.1-1~maverick2_amd64.deb'
        )
        self.check_not_exists(
            'public/pool/main/g/gnuplot/gnuplot-nox_4.6.1-1~maverick2_i386.deb'
        )
        self.check_not_exists(
            'public/pool/main/g/gnuplot/gnuplot-nox_4.6.1-1~maverick2_amd64.deb'
        )

        # verify contents except of sums
        self.check_file_contents(
            'public/dists/maverick/Release', 'release', match_prepare=strip_processor
        )
        self.check_file_contents(
            'public/dists/maverick/main/binary-i386/Packages',
            'binary',
            match_prepare=lambda s: '\n'.join(sorted(s.split('\n'))),
        )


class AzurePublish4Test(AzureTest):
    """
    publish to Azure: multiple repos, list
    """

    fixtureCmds = [
        'aptly repo create -distribution=maverick local-repo',
        'aptly repo add local-repo ${udebs}',
        'aptly publish repo -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec local-repo azure:test1:',
        'aptly publish repo -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec -distribution=xyz local-repo azure:test1:',
        'aptly publish repo -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec local-repo azure:test1:prefix',
    ]
    runCmd = 'aptly publish list'


class AzurePublish5Test(AzureTest):
    """
    publish to Azure: publish drop - component cleanup
    """

    fixtureCmds = [
        'aptly repo create local1',
        'aptly repo create local2',
        'aptly repo add local1 ${files}/libboost-program-options-dev_1.49.0.1_i386.deb',
        'aptly repo add local2 ${files}',
        'aptly publish repo -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec -distribution=sq1 local1 azure:test1:',
        'aptly publish repo -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec -distribution=sq2 local2 azure:test1:',
    ]
    runCmd = 'aptly publish drop sq2 azure:test1:'

    def check(self):
        super(AzurePublish5Test, self).check()

        self.check_exists('public/dists/sq1')
        self.check_not_exists('public/dists/sq2')
        self.check_exists('public/pool/main/')

        self.check_not_exists('public/pool/main/p/pyspi/pyspi_0.6.1-1.3.dsc')
        self.check_not_exists('public/pool/main/p/pyspi/pyspi_0.6.1-1.3.diff.gz')
        self.check_not_exists('public/pool/main/p/pyspi/pyspi_0.6.1.orig.tar.gz')
        self.check_not_exists('public/pool/main/p/pyspi/pyspi-0.6.1-1.3.stripped.dsc')
        self.check_exists(
            'public/pool/main/b/boost-defaults/libboost-program-options-dev_1.49.0.1_i386.deb'
        )
